
import os
import sys
import pandas as pd
from sklearn.metrics import roc_curve, roc_auc_score
from sklearn.preprocessing import LabelEncoder
import matplotlib.pyplot as plt
from scipy.stats import norm


class classifier2():
    """
    Naive Bayes class implementation
    """
    def __init__(self):
        self.lowSalaryPrior = None
        self.highSalaryPrior = None
        self.education_table = None
        self.salary_table = None
        self.workclass_table = None
        
        
    def fit(self, X, Y):
        """
        Naive Bayes training method.
        input X: pandas data frame of feature values. To extract a feature: E.g.: training.loc[:,['education']]
        input Y: pandas series of output values. E.g.: training['salary']
        """
        
        # start with the a-priori probabilities
        self.salary_table = Y.value_counts()
        
        # p(<=50K)
        self.lowSalaryPrior = float(self.salary_table.loc['<=50K'].astype('float')/self.salary_table.sum())
        
        # p(>50K)
        self.highSalaryPrior = float(self.salary_table.loc['>50K'].astype('float')/self.salary_table.sum())
        
        # Education
        self.education_table = pd.crosstab(X['education'], Y)
        
        # Workclass
        self.workclass_table = pd.crosstab(X['workclass'], Y)
        
        # Education number
        self.education_number_table = pd.crosstab(X['education-num'], Y)
        
        # marital status
        self.marital_table = pd.crosstab(X['marital-status'], Y)
        
        # occupation
        self.occupation_table = pd.crosstab(X['occupation'], Y)
        
        # relationship
        self.relationship_table = pd.crosstab(X['relationship'], Y)
        
        # race
        self.race_table = pd.crosstab(X['race'], Y)
        
        # sex
        self.sex_table = pd.crosstab(X['sex'], Y)
        
        # native country
        self.native_country_table = pd.crosstab(X['native-country'], Y)
        
        # age mean and age standard deviation for each salary class
        data_lowSalary = X.loc[X['salary']=="<=50K",:]
        self.age_mean_lowSalary, self.age_sd_lowSalary = data_lowSalary['age'].mean(), data_lowSalary['age'].std()
        
        data_highSalary = X.loc[X['salary']==">50K",:]
        self.age_mean_highSalary, self.age_sd_highSalary = data_highSalary['age'].mean(), data_highSalary['age'].std()

    
    
    def predict(self, x, printScores = True):
        """
        Naive Bayes predicting function
        input x: vector of features corresponding to a row of a pandas data frame
        """
        lowSalaryScore = self.lowSalaryPrior
        highSalaryScore = self.highSalaryPrior
        
        # features
        education = x['education']
        workclass = x['workclass']
        age = x['age']
        education_number = x['education-num']
        marital_status = x['marital-status']
        occupation = x['occupation']
        relationship = x['relationship']
        race = x['race']
        sex = x['sex']
        native_country = x['native-country']
        
        
        if education != '?':
            # TODO: use data in education_table to compute new scores for
            #   lowSalaryScore and highSalaryScore
            #
            # Hints:
            #    number of observations for a particular education value, and for a salary of <=50K: self.education_table.loc[education, '<=50K'].astype('float')
            #    Number of people earning <= 50K: self.salary_table.loc['<=50K']
            #    Convert your final likelihood to float using float(...)
            
            
            likelihood_educationlowSalary = float(self.education_table.loc[education, '<=50K'].astype('float') / self.salary_table.loc['<=50K'])
            likelihood_educationhighSalary = float(self.education_table.loc[education, '>50K'].astype('float') / self.salary_table.loc['>50K'])
        
            lowSalaryScore = lowSalaryScore * likelihood_educationlowSalary
            highSalaryScore = highSalaryScore * likelihood_educationhighSalary
        
        if workclass != '?':
            # workclass
            # TODO: use data in workclass_table to compute new scores for
            #   lowSalaryScore and highSalaryScore
            #
            # Hints:
            #    number of observations for a particular workclass value, and for a salary of <=50K: self.education_table.loc[education, '<=50K'].astype('float')
            #    Number of people earning <= 50K: self.salary_table.loc['<=50K']
            #    Convert your final likelihood to float using float(...)

            likelihood_workclasslowSalary = float(self.workclass_table.loc[workclass, '<=50K'].astype('float') / self.salary_table.loc['<=50K'])
            likelihood_workclasshighSalary = float(self.workclass_table.loc[workclass, '>50K'].astype('float') / self.salary_table.loc['>50K'])
        
            lowSalaryScore = lowSalaryScore * likelihood_workclasslowSalary
            highSalaryScore = highSalaryScore * likelihood_workclasshighSalary
        
        if age!= '?':
            likelihood_agelowSalary = norm.pdf(age, loc=self.age_mean_lowSalary, scale=self.age_sd_lowSalary)
            likelihood_agehighSalary = norm.pdf(age, loc=self.age_mean_highSalary, scale=self.age_sd_highSalary)        
        
            lowSalaryScore = lowSalaryScore * likelihood_agelowSalary
            highSalaryScore = highSalaryScore * likelihood_agehighSalary
            
        if education_number !='?':
            lowSalaryScore = lowSalaryScore * float(self.education_number_table.loc[education_number, '<=50K'].astype('float') / self.salary_table.loc['<=50K'])
            highSalaryScore = highSalaryScore * float(self.education_number_table.loc[education_number, '>50K'].astype('float') / self.salary_table.loc['>50K'])
            
        if marital_status !='?':
            lowSalaryScore = lowSalaryScore * float(self.marital_table.loc[marital_status, '<=50K'].astype('float') / self.salary_table.loc['<=50K'])
            highSalaryScore = highSalaryScore * float(self.marital_table.loc[marital_status, '>50K'].astype('float') / self.salary_table.loc['>50K'])

        if occupation !='?':
            lowSalaryScore = lowSalaryScore * float(self.occupation_table.loc[occupation, '<=50K'].astype('float') / self.salary_table.loc['<=50K'])
            highSalaryScore = highSalaryScore * float(self.occupation_table.loc[occupation, '>50K'].astype('float') / self.salary_table.loc['>50K'])

        if relationship !='?':
            lowSalaryScore = lowSalaryScore * float(self.relationship_table.loc[relationship, '<=50K'].astype('float') / self.salary_table.loc['<=50K'])
            highSalaryScore = highSalaryScore * float(self.relationship_table.loc[relationship, '>50K'].astype('float') / self.salary_table.loc['>50K'])
                            
        if race !='?':
            lowSalaryScore = lowSalaryScore * float(self.race_table.loc[race, '<=50K'].astype('float') / self.salary_table.loc['<=50K'])
            highSalaryScore = highSalaryScore * float(self.race_table.loc[race, '>50K'].astype('float') / self.salary_table.loc['>50K'])

        if sex !='?':
            lowSalaryScore = lowSalaryScore * float(self.sex_table.loc[sex, '<=50K'].astype('float') / self.salary_table.loc['<=50K'])
            highSalaryScore = highSalaryScore * float(self.sex_table.loc[sex, '>50K'].astype('float') / self.salary_table.loc['>50K'])

        if native_country !='?':
            lowSalaryScore = lowSalaryScore * float(self.native_country_table.loc[native_country, '<=50K'].astype('float') / self.salary_table.loc['<=50K'])
            highSalaryScore = highSalaryScore * float(self.native_country_table.loc[native_country, '>50K'].astype('float') / self.salary_table.loc['>50K'])



        
        if printScores:
            print("<=50K: {}, >50K: {}".format(lowSalaryScore, highSalaryScore))
        
        # Return score ratio
        result = highSalaryScore/lowSalaryScore
        return(result)


